#ifndef _WIN32_H_
#define _WIN32_H_

//////////////////////////////////////////////////////////////////
// WIN32.H							//
// Written by Tang Hong October-				//
// Wuxi HighTech Electronics Co.,Ltd.				//
// This chapter require Microsoft Visual C++ 4.0 or later	//
//////////////////////////////////////////////////////////////////

// Error number defination
#define	ERR_FILEOPEN		1001
#define	ERR_FILEMAPPING		1002
#define	ERR_MAPVIEWOFFILE	1003
#define	ERR_UNKNOWNSIGNATURE	1004
#define	ERR_NOTPEFILE		1005
#define	ERR_NOADDITIONALROOM	1006
#define	ERR_CRASHEDRESOURCE	1007
#define	ERR_UNEXPECTED		1008
#define	ERR_NOIMPORTSEGMENT	1009
#define	ERR_OUTOFMEMORY		1010
#define	ERR_FREEMEMORY		1011
#define ERR_NORELOCATION	1012
#define	ERR_NOCODESEGMENT	1013
#define	ERR_INVALIDRELOCATION	1014
#define	ERR_NOTEXECUTABLE	1015
#define	ERR_DEBUGNOTSTRIPPED	1016
#define	ERR_MULTICODESEGMENT	1017
#define	ERR_OPENTARGET		1018
#define	ERR_WRITETARGET		1019
#define	ERR_INVALIDFILESIZE	1020


// PE header offset
#define	NTHDROFFSET(a)	((PIMAGE_NT_HEADERS)((BYTE*)a +	\
			((PIMAGE_DOS_HEADER)a)->e_lfanew))


// Current NUM_IMPORTADDED only be 1, and must be 1
#define	NUM_IMPORTADDED	2

#define	HIGH_ENCRYPT	((DWORD)0x54616E67)
#define	LOW_ENCRYPT	((DWORD)0x486F6E67)

#pragma	pack(1)


typedef	struct	{
	WORD	Hint;
	UCHAR	Name[14];
} IMPORTBYNAME, *PIMPORTBYNAME;

// Structure added at the end of .idata segment
typedef	struct	{
	// Original Thunk Data added,
	// Note: Must import by ORDINAL
	DWORD		OriginalThunkData;
	// Separator
	DWORD		EOFThunkData;
	// Import Address Table added,
	// Note: 
	// IAT original value has no meaning when the program
	// loaded under Windows 95, so it can be NULL,
	// but Windows NT use this value to fill the actual
	// imported address. Both envionment don't need the IAT
	// continuous.
	DWORD		ImportAddressTable;
	// Separator
	DWORD		EOFAddressTable;
	// Import DLL name, don't use long file name
	UCHAR		ImportDLLName[16];
	IMPORTBYNAME	ImportFunction;
} IMPORT_ADDED_STRUCT, *PIMPORT_ADDED_STRUCT;

// The following completes the defination for Base Relocation
typedef	struct	{
	WORD	Offset:12;
	WORD	Type:4;
} IMAGE_RELOCATION_ITEM, *PIMAGE_RELOCATION_ITEM;

// Structure added in .reloc segment
typedef	struct	{
	IMAGE_BASE_RELOCATION	BaseRelocation;
	IMAGE_RELOCATION_ITEM	RelocationItem[NUM_IMPORTADDED];
} RELOCATION_ADDED_STRUCT, *PRELOCATION_ADDED_STRUCT;

// The following define the private structure for this program
typedef	struct	{
	LPVOID	OriginalPointerToRawData;
	DWORD	OriginalVirtualSize;
	LPVOID	PointerToRawData;
	DWORD	VirtualSize;
	DWORD	SizeOfRawData;
} INFO_CHANGED_SEGMENT, *PINFO_CHANGED_SEGMENT;


// The following define the instruction in startcode
typedef	struct	{
	BYTE	instr_push;
	DWORD	instr_data;
} PUSH_INSTRUC, *PPUSH_INSTRUC;

typedef	struct	{
	BYTE	instr_jmp;
	long	OriginalEntry;
} JMP_INSTRUC, *PJMP_INSTRUC;

typedef	struct	{
	BYTE	instr_cmp[3];
	DWORD	RetFromBox;
} CMP_INSTRUC, *PCMP_INSTRUC;

typedef	struct	{
	CHAR		retdata[4];
	PUSH_INSTRUC	data;
	PUSH_INSTRUC	sw;
	PUSH_INSTRUC	id;
	PUSH_INSTRUC	no;
	PUSH_INSTRUC	area;
	PUSH_INSTRUC	did;
	PUSH_INSTRUC	comp;
	PUSH_INSTRUC	commno;
} BOX_ARGUMENTS, *PBOX_ARGUMENTS;

typedef	struct	{
	DWORD	offset_callbox;
	DWORD	offset_entrypoint;
	DWORD	offset_retfrombox;
} DATASEG_STRUCT, *PDATASEG_STRUCT;

#pragma	pack()

#define	KEY_DECRYPT	(HIGH_ENCRYPT + LOW_ENCRYPT)

typedef	struct	{
	LPCSTR	lpszTargetFile;
	int	commno;
	int	comp;
	int	did;
	int	area;
	int	no;
	int	id;
	int	sw;
	int	data;
} MODULE_ARGUMENTS, *PMODULE_ARGUMENTS;

#endif	// _WIN32_H_
